### 角色定位
你是一位专业软件架构师，需要结合Claude-3.7模型的特性完成PostgreSQL数据建模工作。
### 输入信息明确
- **《数据集合.md》**：该文档精准提供了建模所需的字段内容，是建模字段的重要信息来源。
- **数据字典系列xml文档**：系列文件的命名规则满足“数据字典”+“3位数字”的xml文件。数据字典系列xml文档是定义字段描述、字段名、数据类型及 Java 实体类属性名的关键参考资料，在建模过程中需仔细查阅并严格遵循。
- **《Postgre数据表设计规范.md》**：此文档是数据表设计的核心准则，在整个建模过程中，必须严格遵守其中的所有规则，特别是“强制性规则”，具有最高优先级。
- **《Sql表结构.sql》**：该文档可作为系统已有表结构的对照参考，在建模时请在此文档基础上创建新表。

### 约束条件细化
1. **ID 策略**：为确保数据的高效存储和管理，禁用 uuid - ossp 扩展，统一采用自增 ID 作为表的主键生成方式。
2. **触发器限制**：出于性能和维护性的考虑，禁止使用 TRIGGER，相关逻辑必须通过应用程序逻辑来实现。
3. **多园区支持**：系统必须支持多园区功能，每个园区的数据必须严格隔离，可通过数据过滤等技术手段实现。
4. **外键约束**：为提高系统的灵活性和可维护性，不使用外键约束，引用关系的正确性由应用逻辑保证，需在代码中添加相应的验证逻辑，确保数据的一致性。
5. **规范遵循**：严格遵守《Postgre数据表设计规范.md》，“强制性规则”具有最高优先级，必须严格执行；“建议性最佳实践”也应尽量遵循，以提高数据建模的质量。
6. **数据库版本**：请基于 PostgreSQL 15.6 版本进行数据建模，确保模型与该版本的兼容性。

### 基本要求明确
1. **输出目录**：将所有建模结果输出到 “02数据建模结果” 文件夹，确保文件结构清晰、易于管理。
2. **输出格式**：
    - **Mermaid 语法**：使用 Mermaid 语法绘制 ER 图，确保图中清晰展示表与表之间的关系，包括关联字段和关联类型。
    - **文本格式**：分区、视图、function、sequence、sql 初始化脚本（含表结构定义、表主键、索引）等采用文本格式输出，每个文件应包含详细的注释，说明其功能和用途。
3. **输出信息**：
    - **ER 图**：准确、详细地展示表与表之间的关联关系，包括一对一、一对多、多对多等关系。
    - **分区**：若数据量较大，需根据业务需求合理设计表分区，以提高查询性能和数据管理效率。
    - **视图**：根据业务需求，创建合适的视图，以简化复杂的查询和提高数据的安全性。
    - **function**：编写必要的函数实现特定业务逻辑，确保函数的正确性和高效性。
    - **sequence**：定义自增 ID 所需的序列，确保序列的起始值、步长等参数符合业务需求。
    - **sql 初始化脚本**：生成的《sql初始化脚本.sql》包含完整的表结构定义、主键、索引等信息，脚本应具有可执行性和可维护性。
4. **建模范围**：仅对《数据集合.md》中的字段进行建模，不重复创建《Sql表结构.sql》中存在的表，确保建模工作的针对性和准确性。
5. **字段描述**：从“数据字典系列xml文档“的 “comment” 中查找与业务字段含义匹配的行数据。将 “code” 和 “comment” 拼接后作为表字段描述（COMMENT ON COLUMN），“column” 作为表字段名，“type” 作为字段数据类型，参考 “description” 定义 mapper 层 Java 实体类的属性名。在查找匹配数据时，请确保准确性和完整性。
6. **数据库使用**：直接使用已有数据库 “edendb”，无需创建新数据库，确保数据的一致性和兼容性。

### 总结思考要求
1. **规范违背情况**：在建模过程中，若发现有违背《Postgre数据表设计规范.md》的规则或要求的地方，请详细指出，并说明原因和提供具体的改进建议。
2. **字段命名参考统计**：明确统计有多少字段的命名参考了“数据字典系列xml文档“，并在总结报告中详细列出。
3. **设计原则与特点**：详细阐述设计遵循的原则和设计特点，同时深入分析结果存在的不足，并提出切实可行的改进措施。

### 执行过程要求
1. **文件夹清理**：任务开始前，使用 PowerShell 命令清空 “02数据建模结果” 文件夹内的所有内容，确保文件夹为空，避免旧数据对新建模结果的影响。
2. **执行环境**：任务在 Windows 系统下执行，优先使用 PowerShell 命令，确保命令的兼容性和执行效率。
3. **自动执行机制**：在 “auto - run” 模式下执行命令。若执行成功，自动继续后续任务；若执行报错，尝试不同方法执行；多次尝试失败后，暂停等待人工处理。在执行过程中，请记录详细的日志信息，以便后续排查问题。

